@@ -48,7 +48,7 @@ type mpstat = {
   mp_vfs : string;                      (* VFS type (eg. "ext4") *)
 }
 
-let rec convert dir options source =
+let rec convert input_disks options source =
   let target_nics = List.map (Networks.map options.network_map) source.s_nics in
 
   message (f_"Opening the source");
@@ -62,28 +62,38 @@ let rec convert dir options source =
    * ID:clevis" command line options (if any). *)
   g#set_network true;
   List.iter (
-    fun { s_disk_id = i } ->
+    fun uri ->
       (* NB: Old virt-v2v used copyonread here, when it was using a
        * qcow2 file as overlay.  We MUST NOT use copyonread!  It
        * doesn't do anything if there is no backing chain, but worse
        * than that I observed a huge (33x!) slow down.
        *)
-      let socket = sprintf "unix:%s/in%d" dir i in
-      g#add_drive_opts ""
+      let socket, export =
+        match uri with NBD_URI.Unix (socket, export) ->
+          sprintf "unix:%s" socket, Option.value export ~default:"" in
+      g#add_drive_opts export
         ~format:"raw" ~protocol:"nbd" ~server:[| socket |]
         ~cachemode:"unsafe" ~discard:"besteffort"
-  ) source.s_disks;
+  ) input_disks;
 
   g#launch ();
 
   (* Decrypt the disks. *)
   inspect_decrypt g options.ks;
 
-  (* Inspection - this also mounts up the filesystems. *)
-  message (f_"Inspecting the source");
-  let inspect = Inspect_source.inspect_source options.root_choice g in
+  (* Check (fsck) the filesystems before conversion. *)
+  message (f_"Checking filesystem integrity before conversion");
+  do_fsck ~before:true g;
+
+  (* Detect firmware. *)
+  message (f_"Detecting if this guest uses BIOS or UEFI to boot");
   let i_firmware = Firmware.detect_firmware g in
 
+  (* Inspect the source, choose root and mount up the filesystems. *)
+  message (f_"Inspecting the source");
+  let root = Choose_root.choose_root options.root_choice g in
+  let inspect = Mount_filesystems.mount_filesystems g root in
+
   let mpstats = get_mpstats g in
   check_guest_free_space inspect mpstats;
 
@@ -104,6 +114,11 @@ let rec convert dir options source =
   message (f_"Mapping filesystem data to avoid copying unused and blank areas");
   do_fstrim g inspect;
 
+  (* Check (fsck) the filesystems after conversion. *)
+  g#umount_all ();
+  message (f_"Checking filesystem integrity after conversion");
+  do_fsck g;
+
   message (f_"Closing the overlay");
   g#umount_all ();
   g#shutdown ();
@@ -217,19 +232,66 @@ and do_fstrim g inspect =
       )
   ) fses
 
+(* Perform fsck before and after conversion.
+ *
+ * If fsck returns an error then the conversion will fail.  We do not
+ * attempt to do any repairs.
+ *)
+and do_fsck ?(before=false) g =
+  let fses = g#list_filesystems () in
+  List.iter (function
+      | dev, _ when String.starts_with "btrfsvol:" dev ->
+         (* Ignore btrfs volumes, since we should see the btrfs device
+          * somewhere else in the list and checking that is sufficient.
+          *)
+         ()
+
+      | dev, "btrfs" ->
+         (* btrfs-check would be the obvious thing, but the general
+          * opinion seems to be that it's broken, and you should use
+          * btrfs scrub instead.
+          *)
+         Fun.protect ~finally:g#umount_all (
+           fun () ->
+             g#mount_ro dev "/";
+             g#btrfs_scrub_full "/" ~readonly:true;
+         );
+
+      | dev, "ext4" ->
+         g#e2fsck ~forceno:true dev
+
+      | dev, "xfs" ->
+         if before then (
+           (* xfs_repair cannot replay the dirty log, only the kernel can,
+            * so we must first mount then unmount the filesystem, and then
+            * we can run xfs_repair.  Unlike what is documented, xfs_repair
+            * doesn't return 2 in this case.  Mount r/o is fine as that
+            * will still replay the log (RHEL-95365)
+            *)
+           Fun.protect ~finally:g#umount_all (fun () -> g#mount_ro dev "/");
+         );
+
+         if g#xfs_repair ~nomodify:true dev <> 0 then
+           error (f_"detected errors on the XFS filesystem on %s") dev
+
+      | _, _ ->
+         (* Ignore other filesystem types. *)
+         ()
+  ) fses
+
 (* Conversion. *)
 and do_convert g source inspect i_firmware
                block_driver keep_serial_console interfaces =
-  (match inspect.i_product_name, inspect.i_osinfo with
-  | "unknown", "unknown" ->
-    message (f_"Converting the guest to run on KVM")
-  | "unknown", osinfo ->
-    message (f_"Converting %s guest to run on KVM") osinfo
-  | prod, "unknown" ->
-    message (f_"Converting %s to run on KVM") prod
-  | prod, osinfo ->
-    message (f_"Converting %s (%s) to run on KVM") prod osinfo
-  );
+  (* Create the "Converting..." message.  Complicated! *)
+  let () =
+    let what_guest =
+      match inspect.i_product_name, inspect.i_osinfo with
+      | "unknown", "unknown" -> s_"the guest"
+      | "unknown", osinfo -> sprintf (f_"%s guest") osinfo
+      | prod, "unknown" -> prod
+      | prod, osinfo -> sprintf "%s (%s)" prod osinfo in
+
+    message (f_"Converting %s to run on KVM") what_guest in
 
   let convert, conversion_name =
     match inspect with
@@ -299,10 +361,27 @@ and debug_info source inspect
   List.iter (fun nic -> eprintf "%s\n" (string_of_source_nic nic))
     target_nics;
   eprintf "mountpoint stats:\n";
+  eprintf "%20s %-16s %-16s %-16s %s\n" "" "Size" "Used" "Available" "Use%";
   List.iter debug_mpstat mpstats;
+  flush Stdlib.stderr
 
+(* The calculations here are similar to virt-df df/output.c *)
 and debug_mpstat { mp_dev = dev; mp_path = path;
-                   mp_statvfs = s; mp_vfs = vfs } =
-  eprintf "    mountpoint statvfs %s %s (%s):\n" dev path vfs;
-  eprintf "        bsize=%Ld blocks=%Ld bfree=%Ld bavail=%Ld\n"
-    s.Guestfs.bsize s.Guestfs.blocks s.Guestfs.bfree s.Guestfs.bavail
+                   mp_statvfs = { G.bsize; G.blocks; G.bfree; G.bavail };
+                   mp_vfs = vfs } =
+  let label = sprintf "%s %s (%s):" dev path vfs
+  and size = blocks *^ bsize
+  and used = (blocks -^ bfree) *^ bsize
+  and avail = bavail *^ bsize
+  and percent =
+    if blocks <> 0_L then
+      100. -. 100. *. (Int64.to_float bfree /. Int64.to_float blocks)
+    else
+      0. in
+  if String.length label > 20 then
+    eprintf "%s\n%20s " label ""
+  else
+    eprintf "%-20s " label;
+  eprintf "%-16Ld %-16Ld %-16Ld\n" size used avail;
+  eprintf "%20s %-16s %-16s %-16s %.1f%%\n"
+    "" (human_size size) (human_size used) (human_size avail) percent
