@@ -25,14 +25,6 @@ open Tools_utils
 open Types
 open Utils
 
-let error_unless_nbdkit_version_ge min_version =
-  let version = Nbdkit.version () in
-  if version < min_version then (
-    let min_major, min_minor, min_release = min_version in
-    error (f_"nbdkit is too old.  nbdkit >= %d.%d.%d is required.")
-          min_major min_minor min_release
-  )
-
 (* Create an nbdkit module specialized for reading from Curl sources. *)
 let create_curl ?bandwidth ?cookie_script ?cookie_script_renew ?cor
                 ?(sslverify=true) url =
@@ -45,11 +37,6 @@ let create_curl ?bandwidth ?cookie_script ?cookie_script_renew ?cor
   if not (Nbdkit.probe_filter "cow") then
     error (f_"nbdkit-cow-filter is not installed or not working");
 
-  (* The cookie* parameters require nbdkit 1.22, so check that early. *)
-  if cookie_script <> None || cookie_script_renew <> None then (
-    error_unless_nbdkit_version_ge (1, 22, 0)
-  );
-
   (* Construct the nbdkit command. *)
   let cmd = Nbdkit.create "curl" in
   Nbdkit.add_arg cmd "url" url;
@@ -71,11 +58,6 @@ let create_curl ?bandwidth ?cookie_script ?cookie_script_renew ?cor
    *)
   Nbdkit.add_filter_if_available cmd "retry";
 
-  (* Caching extents speeds up qemu-img, especially its consecutive
-   * block_status requests with req_one=1.
-   *)
-  Nbdkit.add_filter_if_available cmd "cacheextents";
-
   (* IMPORTANT! Add the COW filter.  It must be furthest away
    * except for the rate filter.
    *)
