@@ -48,8 +48,6 @@ let rec main () =
     else output_file := Some filename
   in
 
-  let bandwidth = ref None in
-  let bandwidth_file = ref None in
   let input_conn = ref None in
   let input_format = ref None in
   let input_password = ref None in
@@ -135,34 +133,24 @@ let rec main () =
         if_prefix_length; if_nameservers }
   in
 
-  let root_choice = ref AskRoot in
-  let set_root_choice = function
-    | "ask" -> root_choice := AskRoot
-    | "single" -> root_choice := SingleRoot
-    | "first" -> root_choice := FirstRoot
-    | dev when String.is_prefix dev "/dev/" -> root_choice := RootDev dev
-    | s ->
-      error (f_"unknown --root option: %s") s
-  in
+  let root_choice = ref default_root_choice in
+  let set_root_choice = Types.set_root_choice root_choice in
 
-  let input_mode = ref `Not_set in
+  let input_modes =
+    Select_input.input_modes |>
+    List.map Select_input.string_of_input_mode |>
+    String.concat "|" in
+  let input_mode = ref None in
   let set_input_mode mode =
-    if !input_mode <> `Not_set then
+    if !input_mode <> None then
       error (f_"%s option used more than once on the command line") "-i";
-    match mode with
-    | "disk" | "local" -> input_mode := `Disk
-    | "libvirt" -> input_mode := `Libvirt
-    | "libvirtxml" -> input_mode := `LibvirtXML
-    | "ova" -> input_mode := `OVA
-    | "vmx" -> input_mode := `VMX
-    | s ->
-       error (f_"unknown -i option: %s") s
+    input_mode := Some (Select_input.input_mode_of_string mode)
   in
 
   let argspec = [
     [ S 'b'; L"bridge" ], Getopt.String ("in:out", add_bridge),
                                     s_"Map bridge ‘in’ to ‘out’";
-    [ S 'i' ],       Getopt.String ("disk|libvirt|libvirtxml|ova|vmx", set_input_mode),
+    [ S 'i' ],       Getopt.String (input_modes, set_input_mode),
                                     s_"Set input mode (default: libvirt)";
     [ M"ic" ],       Getopt.String ("uri", set_string_option_once "-ic" input_conn),
                                     s_"Libvirt URI";
@@ -231,8 +219,8 @@ read the man page virt-v2v-inspector(1).
   let input_transport =
     match !input_transport with
     | None -> None
-    | Some "ssh" -> Some `SSH
-    | Some "vddk" -> Some `VDDK
+    | Some "ssh" -> Some Input.SSH
+    | Some "vddk" -> Some Input.VDDK
     | Some transport ->
        error (f_"unknown input transport ‘-it %s’") transport in
   let root_choice = !root_choice in
@@ -250,11 +238,9 @@ read the man page virt-v2v-inspector(1).
       pr "mac-option\n";
       pr "mac-ip-option\n";
       pr "customize-ops\n";
-      pr "input:disk\n";
-      pr "input:libvirt\n";
-      pr "input:libvirtxml\n";
-      pr "input:ova\n";
-      pr "input:vmx\n";
+      Select_input.input_modes |>
+        List.map Select_input.string_of_input_mode |>
+        List.iter (pr "input:%s\n");
       pr "convert:linux\n";
       pr "convert:windows\n";
       List.iter (pr "ovf:%s\n") Create_ovf.ovf_flavours;
@@ -262,62 +248,12 @@ read the man page virt-v2v-inspector(1).
    | _, _ -> ()
   );
 
-  (* Get the input module. *)
+  (* Select the input module. *)
   let (module Input_module) =
-    match input_mode with
-    | `Disk -> (module Input_disk.Disk : Input.INPUT)
-    | `LibvirtXML -> (module Input_libvirt.LibvirtXML)
-    | `OVA -> (module Input_ova.OVA)
-    | `VMX -> (module Input_vmx.VMX)
-    | `Not_set | `Libvirt ->
-       match input_conn with
-       | None -> (module Input_libvirt.Libvirt_)
-       | Some orig_uri ->
-          let { Xml.uri_server = server; uri_scheme = scheme } =
-            try Xml.parse_uri orig_uri
-            with Invalid_argument msg ->
-              error (f_"could not parse '-ic %s'.  \
-                        Original error message was: %s")
-                orig_uri msg in
-
-          match server, scheme, input_transport with
-          | None, _, _
-            | Some "", _, _       (* Not a remote URI. *)
-
-            | Some _, None, _     (* No scheme? *)
-            | Some _, Some "", _ ->
-             (module Input_libvirt.Libvirt_)
-
-          (* vCenter over https. *)
-          | Some server, Some ("esx"|"gsx"|"vpx"), None ->
-             (module Input_vcenter_https.VCenterHTTPS)
-
-          (* vCenter or ESXi using nbdkit vddk plugin *)
-          | Some server, Some ("esx"|"gsx"|"vpx"), Some `VDDK ->
-             (module Input_vddk.VDDK)
-
-          (* Xen over SSH *)
-          | Some server, Some "xen+ssh", _ ->
-             (module Input_xen_ssh.XenSSH)
-
-          (* Old virt-v2v also supported qemu+ssh://.  However I am
-           * deliberately not supporting this in new virt-v2v.  Don't
-           * use virt-v2v if a guest already runs on KVM.
-           *)
-
-          (* Unknown remote scheme. *)
-          | Some _, Some _, _ ->
-             warning (f_"no support for remote libvirt connections \
-                         to '-ic %s'.  The conversion may fail when it \
-                         tries to read the source disks.") orig_uri;
-             (module Input_libvirt.Libvirt_) in
+    Select_input.select_input input_mode input_conn input_transport in
 
   let input_options = {
-    Input.bandwidth =
-      (match !bandwidth, !bandwidth_file with
-       | None, None -> None
-       | Some rate, None -> Some (StaticBandwidth rate)
-       | rate, Some filename -> Some (DynamicBandwidth (rate, filename)));
+    Input.bandwidth = None;
     input_conn = input_conn;
     input_format = !input_format;
     input_options = !input_options;
@@ -354,11 +290,11 @@ read the man page virt-v2v-inspector(1).
   (* Start the input module (runs an NBD server in the background). *)
   message (f_"Setting up the source: %s")
     (Input_module.to_string input_options args);
-  let source = Input_module.setup v2vdir input_options args in
+  let source, input_disks = Input_module.setup v2vdir input_options args in
 
   (* Do the conversion. *)
   with_open_out (v2vdir // "convert") (fun _ -> ());
-  let inspect, target_meta = Convert.convert v2vdir conv_options source in
+  let inspect, target_meta = Convert.convert input_disks conv_options source in
   unlink (v2vdir // "convert");
 
   (* Debug the v2vdir. *)
@@ -368,7 +304,7 @@ read the man page virt-v2v-inspector(1).
   );
 
   (* Dump out the information. *)
-  let doc = create_inspector_xml v2vdir inspect target_meta in
+  let doc = create_inspector_xml input_disks inspect target_meta in
   let chan =
     match output_file with
     | None -> Stdlib.stdout
