@@ -373,9 +373,20 @@ type target_meta = {
 
 (** {2 Command line parameters} *)
 
-type root_choice = AskRoot | SingleRoot | FirstRoot | RootDev of string
+type root_choice =
+  | AskRoot                     (** Ask the user interactively (default) *)
+  | SingleRoot                  (** Fail if multi-boot *)
+  | FirstRoot                   (** First from list returned by libguestfs *)
+  | RootDev of string           (** Named device *)
 (** Type of [--root] (root choice) option. *)
 
+val default_root_choice : root_choice
+(** The default {!root_choice} if no [--root] on the command line. *)
+
+val set_root_choice : root_choice ref -> string -> unit
+(** Parse a string (from the command line) into the {!root_choice} option.
+    The reference (first parameter) is updated. *)
+
 type output_allocation = Sparse | Preallocated
 (** Type of [-oa] (output allocation) option. *)
 
