@@ -33,6 +33,9 @@ val have_selinux : bool
 val uri_quote : string -> string
 (** Take a string and perform %xx escaping as used in some parts of URLs. *)
 
+val fnmatch_escape : string -> string
+(** Escape characters like [?] and [*] which are special for fnmatch(3).  *)
+
 val kvm_arch : string -> string
 (** Map guest architecture found by inspection to the architecture
     that KVM must emulate.  Note for x86 we assume a 64 bit hypervisor. *)
@@ -68,6 +71,9 @@ val chown_for_libvirt_rhbz_1045069 : string -> unit
 
 val error_if_no_ssh_agent : unit -> unit
 
+val nbdcopy_supports_blkhash : unit -> bool
+(** Return true if [nbdcopy] supports the [--blkhash] flag. *)
+
 val create_v2v_directory : unit -> string
 (** Create the directory containing inX and outX sockets. *)
 
@@ -75,23 +81,23 @@ val wait_for_file : string -> int -> bool
 (** [wait_for_file filename timeout] waits up to [timeout] seconds for
     [filename] to appear.  It returns [true] if the file appeared. *)
 
-val with_nbd_connect_unix : ?meta_contexts:string list ->
-                            socket:string ->
-                            (NBD.t -> 'a) ->
-                            'a
-(** [with_nbd_connect_unix socket meta_contexts f] calls function [f] with the
-    NBD server at Unix domain socket [socket] connected, and the metadata
-    contexts in [meta_contexts] requested (each of which is not necessarily
+val with_nbd_connect_uri : ?meta_contexts:string list ->
+                           uri:string ->
+                           (NBD.t -> 'a) ->
+                           'a
+(** [with_nbd_connect_uri uri meta_contexts f] calls function [f] with the
+    NBD server at [uri] connected, and the metadata contexts in
+    [meta_contexts] requested (each of which is not necessarily
     supported by the server though). The connection is torn down either on
     normal return or if the function [f] throws an exception. *)
 
-val get_disk_allocated : dir:string -> disknr:int -> int64 option
-(** Callable only in the finalization step. [get_disk_allocated dir disknr]
-    examines output disk [disknr] through the corresponding NBD server socket
-    that resides in [dir]. Returns the number of bytes allocated in the disk
-    image, according to the "base:allocation" metadata context. If the context
-    is not supported by the NBD server behind the socket, the function returns
-    None. *)
+val get_disk_allocated : string -> int64 option
+(** Examines the disk [uri].  Return the number of bytes allocated
+    in the disk image, according to the "base:allocation" metadata
+    context.  If the context is not supported by the NBD server
+    behind the socket, the function returns None.
+
+    For output disks this is callable only in the finalization step. *)
 
 val get_uefi_arch_suffix : string -> string option
 (** [get_uefi_arch_suffix arch] maps [arch] from [inspect.i_arch] representation
