@@ -18,8 +18,8 @@
 
 (** Functions for making files and directories as another user.
 
-    [-o rhv] output mode has to write files as UID:GID 36:36,
-    otherwise RHV cannot read them.  Because the files are located on
+    [-o ovirt] output mode has to write files as UID:GID 36:36,
+    otherwise oVirt cannot read them.  Because the files are located on
     NFS (and hence might be root-squashed) we also cannot chown the
     files.  We cannot setuid the whole process to 36:36 because it
     needs to do other root things like mounting and unmounting the NFS
