@@ -42,8 +42,9 @@ module type OUTPUT = sig
   val to_string : options -> string
   val query_output_options : unit -> unit
   val parse_options : options -> Types.source -> poptions
-  val setup : string -> poptions -> Types.source -> t
-  val finalize : string -> poptions -> t ->
+  val setup : string -> poptions -> Types.source -> NBD_URI.t list ->
+              t * NBD_URI.t list
+  val finalize : string -> poptions -> t -> NBD_URI.t list ->
                  Types.source -> Types.inspect -> Types.target_meta ->
                  unit
   val request_size : int option
@@ -52,21 +53,15 @@ end
 let error_option_cannot_be_used_in_output_mode mode opt =
   error (f_"-o %s: %s option cannot be used in this output mode") mode opt
 
-let get_disks dir =
-  let rec loop acc i =
-    let socket = sprintf "%s/in%d" dir i in
-    if Sys.file_exists socket then (
-      let size = Utils.with_nbd_connect_unix ~socket NBD.get_size in
-      loop ((i, size) :: acc) (i+1)
-    )
-    else
-      List.rev acc
-  in
-  loop [] 0
+let get_disk_sizes =
+  List.map (
+    fun uri ->
+      let uri = NBD_URI.to_uri uri in
+      Utils.with_nbd_connect_uri ~uri NBD.get_size
+  )
 
-let error_if_disk_count_gt dir n =
-  let socket = sprintf "%s/in%d" dir n in
-  if Sys.file_exists socket then
+let error_if_disk_count_gt input_disks n =
+  if List.length input_disks > n then
     error (f_"this output module doesn't support copying more than %d disks") n
 
 type on_exit_kill = Kill | KillAndWait
@@ -102,10 +97,7 @@ let output_to_local_file ?(changeuid = fun f -> f ()) ?(compressed = false)
     | "raw" ->
        let cmd = Nbdkit.create "file" in
        Nbdkit.add_arg cmd "file" filename;
-       if Nbdkit.version () >= (1, 22, 0) then (
-         let cmd = Nbdkit.add_arg cmd "cache" "none" in
-         cmd
-       );
+       Nbdkit.add_arg cmd "cache" "none";
        let _, pid = Nbdkit.run_unix socket cmd in
        pid
 
@@ -155,3 +147,70 @@ let output_to_local_file ?(changeuid = fun f -> f ()) ?(compressed = false)
 let disk_path os name i =
   let outdisk = sprintf "%s/%s-sd%s" os name (drive_name i) in
   absolute_path outdisk
+
+let create_local_output_disks dir
+      ?(compressed = false)
+      output_alloc output_format output_name output_storage
+      input_disks =
+  let input_sizes = get_disk_sizes input_disks in
+
+  let output_disk_names =
+    List.mapi (fun i _ -> disk_path output_storage output_name i) input_disks in
+
+  (* In the special case where we will write to uncompressed raw
+   * local files, create a single nbdkit instance using the 'dir' option
+   * and choose which file to write using the export name.
+   *)
+  if not compressed && output_format = "raw" then (
+    (* Check nbdkit is installed and has the required plugin. *)
+    if not (Nbdkit.is_installed ()) then
+      error (f_"nbdkit is not installed or not working.  It is required \
+                to use ‘-o disk’.");
+    if not (Nbdkit.probe_plugin "file") then
+      error (f_"nbdkit-file-plugin is not installed or not working");
+
+    (* We still have to create the output disks. *)
+    let g = open_guestfs () in
+    let preallocation =
+      match output_alloc with
+      | Preallocated -> Some "full"
+      | Sparse -> None in
+    List.iter (
+      fun (size, filename) ->
+        g#disk_create ?preallocation filename output_format size
+    ) (List.combine input_sizes output_disk_names);
+
+    let socket = sprintf "%s/out0" dir in
+    On_exit.unlink socket;
+
+    (* Create the single nbdkit-file-plugin instance. *)
+    let cmd = Nbdkit.create "file" in
+    Nbdkit.add_arg cmd "dir" output_storage;
+    Nbdkit.add_arg cmd "cache" "none";
+    let _, pid = Nbdkit.run_unix socket cmd in
+    On_exit.kill pid;
+
+    (* Use NBD export names to select the right disk to write. *)
+    let uris =
+      List.mapi (
+        fun i _ ->
+          let export = sprintf "%s-sd%s" output_name (drive_name i) in
+          NBD_URI.Unix (socket, Some export)
+      ) input_disks in
+
+    uris
+  )
+  else (
+    (* Not the special case, use {!output_to_local_file} on each disk. *)
+    List.mapi (
+      fun i (size, outdisk) ->
+        let socket = sprintf "%s/out%d" dir i in
+        On_exit.unlink socket;
+
+        (* Create the actual output disk. *)
+        output_to_local_file ~compressed output_alloc output_format
+          outdisk size socket;
+
+        NBD_URI.Unix (socket, None)
+    ) (List.combine input_sizes output_disk_names)
+  )
