@@ -47,16 +47,17 @@ module type OUTPUT = sig
       stores the parsed information and is passed to both
       {!setup} and {!finalize} methods. *)
 
-  val setup : string -> poptions -> Types.source -> t
-  (** [setup dir poptions source]
+  val setup : string -> poptions -> Types.source -> NBD_URI.t list ->
+              t * NBD_URI.t list
+  (** [setup dir poptions source input_disks]
 
       Set up the output mode.  Sets up a disk pipeline
-      [dir // "outX"] for each output disk. *)
+      for each output disk, and returns the list of output disks. *)
 
-  val finalize : string -> poptions -> t ->
+  val finalize : string -> poptions -> t -> NBD_URI.t list ->
                  Types.source -> Types.inspect -> Types.target_meta ->
                  unit
-  (** [finalize dir poptions t inspect target_meta]
+  (** [finalize dir poptions t output_disks inspect target_meta]
 
       Finalizes the conversion and writes metadata. *)
 
@@ -72,16 +73,27 @@ val error_option_cannot_be_used_in_output_mode : string -> string -> unit
 (** [error_option_cannot_be_used_in_output_mode mode option]
     prints error message that option cannot be used in this output mode. *)
 
-val get_disks : string -> (int * int64) list
-(** Examines the v2v directory and opens each input socket (in0 etc),
-    returning a list of input disk index and size. *)
+val get_disk_sizes : NBD_URI.t list -> int64 list
+(** Call NBD.get_size on each input disk. *)
 
-val error_if_disk_count_gt : string -> int -> unit
+val error_if_disk_count_gt : NBD_URI.t list -> int -> unit
 (** This function lets an output module enforce a maximum disk count.
-    [error_if_disk_count_gt dir n] checks whether the domain has more than [n]
-    disks that need to be copied, by examining the existence of input NBD socket
-    "in[n]" in the v2v directory [dir].  If the socket exists, [error] is
-    called. *)
+
+    [error_if_disk_count_gt input_disks n] checks whether the domain
+    has more than [n] disks that need to be copied. *)
+
+val create_local_output_disks : string ->
+                                ?compressed:bool ->
+                                Types.output_allocation ->
+                                string -> string -> string ->
+                                NBD_URI.t list ->
+                                NBD_URI.t list
+(** When an output mode wants to create one output disk per
+    input disk, using the names "<output_storage>/<output_name>-sdX",
+    this function does it all, taking a list of input NBD URIs and
+    returning a list of output NBD URIs.
+
+    For anything more complicated, use {!output_to_local_file} instead. *)
 
 type on_exit_kill = Kill | KillAndWait
 
