@@ -72,8 +72,7 @@ module Libvirt_ = struct
     (conn, !compressed, options.output_alloc, options.output_format,
      output_name, output_pool)
 
-  let setup dir options source =
-    let disks = get_disks dir in
+  let setup dir options source input_disks =
     let conn, compressed, output_alloc, output_format,
         output_name, output_pool = options in
     let conn = Lazy.force conn in
@@ -133,20 +132,13 @@ module Libvirt_ = struct
     let pool_name = Libvirt.Pool.get_name (Libvirt.Pool.const pool) in
 
     (* Set up the NBD servers. *)
-    List.iter (
-      fun (i, size) ->
-        let socket = sprintf "%s/out%d" dir i in
-        On_exit.unlink socket;
+    let uris =
+      create_local_output_disks dir ~compressed output_alloc output_format
+        output_name target_path input_disks in
 
-        (* Create the actual output disk. *)
-        let outdisk = target_path // output_name ^ "-sd" ^ (drive_name i) in
-        output_to_local_file ~compressed output_alloc output_format
-          outdisk size socket
-    ) disks;
+    (capabilities_xml, pool_name), uris
 
-    (capabilities_xml, pool_name)
-
-  let rec finalize dir options t source inspect target_meta =
+  let rec finalize dir options t output_disks source inspect target_meta =
     let conn, _, output_alloc, output_format, output_name, output_pool =
       options in
     let capabilities_xml, pool_name = t in
