@@ -80,23 +80,28 @@ module QEMU = struct
     (compressed, qemu_boot, options.output_alloc, options.output_format,
      output_name, output_storage)
 
-  let setup dir options source =
-    let disks = get_disks dir in
+  let setup dir options source input_disks =
+    let input_sizes = get_disk_sizes input_disks in
     let compressed, _, output_alloc, output_format,
         output_name, output_storage = options in
 
-    List.iter (
-      fun (i, size) ->
-        let socket = sprintf "%s/out%d" dir i in
-        On_exit.unlink socket;
+    let uris =
+      List.mapi (
+        fun i size ->
+          let socket = sprintf "%s/out%d" dir i in
+          On_exit.unlink socket;
 
-        (* Create the actual output disk. *)
-        let outdisk = disk_path output_storage output_name i in
-        output_to_local_file ~compressed output_alloc output_format
-          outdisk size socket
-    ) disks
+          (* Create the actual output disk. *)
+          let outdisk = disk_path output_storage output_name i in
+          output_to_local_file ~compressed output_alloc output_format
+            outdisk size socket;
 
-  let finalize dir options () source inspect target_meta =
+          NBD_URI.Unix (socket, None)
+      ) input_sizes in
+
+    (), uris
+
+  let finalize dir options () output_disks source inspect target_meta =
     let _, qemu_boot, output_alloc, output_format,
         output_name, output_storage = options in
 
